% samcost3d.m
% example of the SAM cost function,
% compared to the inverse of the SSNR cost function
% and the related MERRY cost function
%
% Written by Rick Martin, 06/13/2002
% Last updated on 08/08/2002
% Cornell University Blind Equalization Research Group
%
% Copyright 2002 Cornell University, All Rights Reserved
%
% SOFTWARE shall mean "samcost3d.m", Cornell Research Foundation 
% ("CRF") Docket D-3006.  SOFTWARE is made available to allow
% non-commercial and research use of SOFTWARE.  Cornell
% University specifically reserves all commercial rights to
% SOFTWARE and these rights may be licensed by CRF on behalf   
% of Cornell University to third parties.
%
% CRF provides SOFTWARE on an ``as is'' basis.  CRF does not
% warrant, guarantee, or make any representations regarding 
% the use or results of SOFTWARE with respect to its
% correctness, accuracy, reliability or performance.  The
% entire risk of the use and performance of SOFTWARE is  
% assumed by EVALUATOR.  ALL WARRANTIES INCLUDING, WITHOUT
% LIMITATION, ANY WARRANTY OF FITNESS  FOR A PARTICULAR   
% PURPOSE OR MERCHANTABILITY ARE HEREBY EXCLUDED.

% Miscellaneous functions
addpath ../mcmtools

% parameter values and flags
nu = 1;           % CP length (desired channel: length nu+1)
c = [1 0.3 0.2];  % channel
Lc = length(c);   % channel length
Lf = 3;           % equalizer length
plotmerry = 0;    % flag: 1=plot MERRY cost, 0=don't plot
contours = 1;     % flag: 1=plot contours, 0=don't plot
surfaces = 0;     % flag: 1=plot surfaces, 0=don't plot

% number of possible windows of length nu+1:
Nwin = (Lf+Lc-1)-(nu+1 - 1);

% constraint: norm(f) = 1, so use spherical coordinates
% [f0 f1 f2] = [fx fz fy], not [fx fy fz]
% f0 = fx = cos(theta) sin(phi)
% f2 = fy = sin(theta) sin(phi)
% f1 = fz = cos(phi)
theta = [0:2*pi/400:2*pi];
phi = [0:pi/200:pi];
Nt = length(theta);
Np = length(phi);

% allocate memory
Jsam = zeros(Nt,Np);
SSNR = zeros(Nt,Np);
if plotmerry, MERRY = zeros(Nt,Np); end;

% compute costs for each window and each equalizer setting
hh = waitbar(0,'go, Sam, go...');
for win=1:Nwin
   for i=1:Nt
      for j=1:Np
         % determine equalizer, combined channel
         f = [cos(theta(i))*sin(phi(j)), cos(phi(j)),...
              sin(theta(i))*sin(phi(j))];
         h = conv(f,c);

         % Shortening SNR cost for this equalizer
         SSNR(i,j) = Jssnr(h,nu,0,win,win);

         % SAM cost for this equalizer
         Rhh = conv(h,fliplr(h));
         Rhh = Rhh((length(Rhh)+1)/2 : end);  % make it one-sided
         Jsam(i,j) = sum( (Rhh(nu+1 +1 :end)).^2 );

         % MERRY cost for this equalizer
         if plotmerry, MERRY(i,j) = Jssnr(h,nu,1,win,win); end;
      end  % (j loop)
      waitbar((1/Nwin)*(i/Nt)+(win-1)/Nwin,hh);
   end  % (i loop)
   eval(['iSSNR',num2str(win),' = 1./SSNR;']);
   if plotmerry
      eval(['MERRY',num2str(win),' = MERRY;']);
   end
end  % (window number loop)
close(hh);

% compute the "amalgamated" costs
% (i.e. minimize over all possible windows)
hh = waitbar(0,'minimizing over window choices...');
for i=1:Nt
   for j=1:Np
      iSSNR(i,j) = min([iSSNR1(i,j), iSSNR2(i,j), ...
                        iSSNR3(i,j), iSSNR4(i,j)]);
      if plotmerry
         MERRY(i,j) = min([MERRY1(i,j), MERRY2(i,j), ...
                           MERRY3(i,j), MERRY4(i,j)]);
      end
   end
   waitbar(i/Nt,hh);
end
close(hh);

% global minimum of SAM cost
[temp,I1] = min(Jsam);
[temp,J1] = min(temp);
I1 = I1(J1);
t1a = theta(I1); p1a = phi(J1);
% reflected minima
t1b = mod(t1a+pi,2*pi); p1b = mod(pi-p1a,2*pi);
t1c = mod(pi*2*5/4-t1b,2*pi); p1c = p1b;
t1d = mod(t1c+pi,2*pi); p1d = mod(pi-p1c,2*pi);

% global minimum of 1/SSNR cost
[temp,I2] = min(iSSNR);
[temp,J2] = min(temp);
I2 = I2(J2);
t2a = theta(I2); p2a = phi(J2);
% reflected minima
t2b = mod(t2a+pi,2*pi); p2b = mod(pi-p2a,2*pi);

% plane of symmetry
thsym = [1/4 1/4 5/4 5/4];
phisym = [0 1 1 0];

if contours
   % levels for contour plots
   % (we cut off the peaks to see more detail in the valleys)
   MMJ = max(max(Jsam));
   mmJ = min(min(Jsam));
   %levels = [0:MMJ/400:MMJ/20];
   levels = logspace(log10(mmJ),log10(MMJ),20);
   MMS = max(max(iSSNR));
   mmS = min(min(iSSNR));
   %levels2 = [0:MMS/400:MMS/10];
   levels2 = logspace(log10(mmS),log10(MMS),20);

   % contour plot of SAM cost
   figure(1);
   clf;
   set(0,'DefaultAxesFontSize',14);
   contour(theta/pi,phi/pi,Jsam.',levels);
   hold on
   plot(t2a/pi,p2a/pi,'ko');  % minima 1 of Jssnr
   plot(t2b/pi,p2b/pi,'ko');  % minima 2 of Jssnr
   plot(thsym,phisym,'k--');  % plane of symmetry
   hold off
   xlabel('\theta / \pi','FontSize',14);
   ylabel('\phi / \pi','FontSize',14);
   title('SAM cost','FontSize',14);

   % contour plot of 1/SSNR, amalgamated
   figure(2);
   clf;
   set(0,'DefaultAxesFontSize',14);
   contour(theta/pi,phi/pi,iSSNR.',levels2);
   hold on
   plot(t1a/pi,p1a/pi,'ko');  % minima of 1 Jsam
   plot(t1b/pi,p1b/pi,'ko');  % minima of 2 Jsam
   plot(t1c/pi,p1c/pi,'ko');  % minima of 3 Jsam
   plot(t1d/pi,p1d/pi,'ko');  % minima of 4 Jsam
   hold off
   xlabel('\theta / \pi','FontSize',14);
   ylabel('\phi / \pi','FontSize',14);
   title('1/SSNR','FontSize',14);

   % contour plots of 1/SSNR for each window
   figure(3);
   clf;
   set(0,'DefaultAxesFontSize',14);
   subplot(2,2,1)
   contour(theta/pi,phi/pi,iSSNR1.',levels2);
   subplot(2,2,2)
   contour(theta/pi,phi/pi,iSSNR2.',levels2);
   subplot(2,2,3)
   contour(theta/pi,phi/pi,iSSNR3.',levels2);
   subplot(2,2,4)
   contour(theta/pi,phi/pi,iSSNR4.',levels2);
   %xlabel('\theta / \pi','FontSize',14);
   %ylabel('\phi / \pi','FontSize',14);
   %title('1/SSNR','FontSize',14);
end  % (contour plots)

if surfaces
   % surface (mesh) plot of SAM cost
   figure(4);
   clf;
   set(0,'DefaultAxesFontSize',14);
   mesh(theta/pi,phi/pi,Jsam.');
   xlabel('\theta / \pi','FontSize',14);
   ylabel('\phi / \pi','FontSize',14);
   title('SAM cost','FontSize',14);

   % surface (mesh) plot of 1/SSNR cost
   figure(5);
   clf;
   set(0,'DefaultAxesFontSize',14);
   mesh(theta/pi,phi/pi,iSSNR.');
   xlabel('\theta / \pi','FontSize',14);
   ylabel('\phi / \pi','FontSize',14);
   title('1/SSNR','FontSize',14);
end

if (surfaces & plotmerry)
   % surface (mesh) plot of MERRY cost
   figure(6);
   clf;
   set(0,'DefaultAxesFontSize',14);
   mesh(theta/pi,phi/pi,MERRY.');
   xlabel('\theta / \pi','FontSize',14);
   ylabel('\phi / \pi','FontSize',14);
   title('MERRY cost','FontSize',14);
end

if 0

figure(1); print -depsc samex2a_co.eps
figure(1); print -deps samex2a_bw.eps
figure(2); print -depsc samex2b_co.eps
figure(2); print -deps samex2b_bw.eps

end
