function [f,Jsam,Jssnr,BR] = samiterar(finit,r,c,params)
% [f,Jsam,Jssnr,BR] = samiterar(finit,r,c,params)
%     Jsam = SAM cost vs. iteration number
%     Jssnr = 1/SSNR vs. iteration number
%     BR = achievable bit rate
%     f = final channel shortening filter (column vector)
%     finit = initialization of channel shortening filter
%     r = received data sequence (row vector)
%     c = channel (set to 1 if unknown...)
%
%     params = [mu,alpha,nu,Lc,cnst,sx,sn,flag1]
%     mu = algorithm step size
%     alpha = AR forgetting factor
%     nu = cyclic prefix length (window length - 1)
%     Lc = number of taps in channel - 1
%     cnst = contraint:
%        0 -> norm(f)=1
%        1 -> norm(h)=1, norm(h) assumed to be known exactly
%        2 -> norm(h)=1, norm(h) blindly estimated from the data
%     sx = sqrt(signal power)
%     sn = sqrt(noise power)
%     flag1: set this to zero to skip performance evaluation
%        0 -> skip non-essential calculations
%        1 -> do the calculations (for Jsam, Jssnr, BR)
%
%     SAM update routine, auto-regressive implementation
%     Written by Rick Martin, 05/22/2002
%     Last updated 06/11/2002
%     Cornell University Blind Equalization Research Group
%
%     Copyright 2002 Cornell University, All Rights Reserved
%
%     SOFTWARE shall mean "samiterar.m", Cornell Research Foundation 
%     ("CRF") Docket D-3006.  SOFTWARE is made available to allow
%     non-commercial and research use of SOFTWARE.  Cornell
%     University specifically reserves all commercial rights to
%     SOFTWARE and these rights may be licensed by CRF on behalf   
%     of Cornell University to third parties.
%
%     CRF provides SOFTWARE on an ``as is'' basis.  CRF does not
%     warrant, guarantee, or make any representations regarding 
%     the use or results of SOFTWARE with respect to its
%     correctness, accuracy, reliability or performance.  The
%     entire risk of the use and performance of SOFTWARE is  
%     assumed by EVALUATOR.  ALL WARRANTIES INCLUDING, WITHOUT
%     LIMITATION, ANY WARRANTY OF FITNESS  FOR A PARTICULAR   
%     PURPOSE OR MERCHANTABILITY ARE HEREBY EXCLUDED.

% unpack the scalar parameters
mu    = params(1);
alpha = params(2);
nu    = params(3);
Lc    = params(4);
cnst  = params(5);
sx    = params(6);
sn    = params(7);
flag1 = params(8);

ssx = sx*sx;
ssn = sn*sn;
Lf = length(finit)-1;
L = Lc + Lf;   % combined channel has L+1 taps
nmax = length(r);
mu2 = 2*mu;
f = finit/norm(finit);

% form first few values of y
y = zeros(size(r));
for n=1:Lf
   y(n) = r(n:-1:1) * finit(1:n);
end
for n=Lf+1:L+Lf
   y(n) = r(n:-1:n-Lf) * f;
end

% allocate memory
A = zeros(L+Lf-nu,1);
B = zeros(L-nu,1);
C = zeros(Lf+1,L-nu);
Jsam = zeros(1,nmax-(L+Lf+1));
Jssnr = zeros(1,nmax-(L+Lf+1));
BR = zeros(1,nmax-(L+Lf+1));
hh = waitbar(0,'Run, SAM, run...');
for n=L+Lf+1:nmax-1
   % compute y(n)
   y(n) = r(n:-1:n-Lf) * f;

   % update A, B, C
   F = convmtx(f',L-nu);
   A = (1-alpha)*A + alpha*y(n)*r(n-nu-1:-1:n-L-Lf).';
   B = F*A;
   C = (1-alpha)*C + alpha*...
        r(n:-1:n-Lf).' * y(n-nu-1:-1:n-L);

   % form summation over l
   acc = zeros(Lf+1,1);  % accumulator
   % (let ll = l - nu)
   for ll=1:L-nu
      acc = acc + B(ll)*(A(ll:ll+Lf) + C(:,ll));
   end

   % update f and renormalize
   f = f - mu2 * acc;
   if cnst == 0
      f = f/norm(f);  % enforce norm(f) = 1
   elseif cnst == 1
      h = conv(c,f);
      f = f/norm(h);  % enforce norm(h) = 1
   else
      normh = sqrt(mean( y(n:n-L-Lf).^2 ));
      f = f/normh;    % enforce norm(h) = 1
   end

   % "non-essential" calculations
   if (flag1 == 1) | (n == nmax-1)
      h = conv(c,f);  % compute the combined channel

      % SAM cost for this equalizer
      Rhh = conv(h,fliplr(h));
      Rhh = Rhh((length(Rhh)+1)/2 : end);  % make it one-sided
      Jsam(n-L-Lf) = sum( (Rhh(nu+1 +1 : end)).^2 );

      % inverse shortening SNR for this equalizer
      Jssnr(n-L-Lf) = 1/ssnr2(h,nu);

      % find window in h with highest energy
      Ph = h.*h;
      sumh = zeros(1,length(Ph)-nu);
      for k=1:length(Ph)-nu;
         sumh(k) = sum(Ph(k:k+nu));
      end
      [dummy dopt] = max(sumh);
      dopt = dopt-1;  % optimal delay

      % achievable bit rate
      BR(n-L-Lf) = sum(bitrate(h,f,nu,dopt,ssx,ssn));
   end  % cost calculations

   waitbar((n+1)/nmax,hh);
end  % n loop
close(hh);
