% samtest2.m
% SAM algorithm test
% Simulates SAM for one or more SNR values, and computes
% the SAM cost, the SSNR, and the bit rate.
%
% Written by Rick Martin, 06/05/2002
% Last updated 06/20/2002
% Cornell University Blind Equalization Research Group
%
% Copyright 2002 Cornell University, All Rights Reserved
%
% SOFTWARE shall mean "samtest2.m", Cornell Research Foundation 
% ("CRF") Docket D-3006.  SOFTWARE is made available to allow
% non-commercial and research use of SOFTWARE.  Cornell
% University specifically reserves all commercial rights to
% SOFTWARE and these rights may be licensed by CRF on behalf   
% of Cornell University to third parties.
%
% CRF provides SOFTWARE on an ``as is'' basis.  CRF does not
% warrant, guarantee, or make any representations regarding 
% the use or results of SOFTWARE with respect to its
% correctness, accuracy, reliability or performance.  The
% entire risk of the use and performance of SOFTWARE is  
% assumed by EVALUATOR.  ALL WARRANTIES INCLUDING, WITHOUT
% LIMITATION, ANY WARRANTY OF FITNESS  FOR A PARTICULAR   
% PURPOSE OR MERCHANTABILITY ARE HEREBY EXCLUDED.

% Miscellaneous functions
addpath ../mcmtools

% DMT TEQ toolbox
addpath ../teqv3

% system parameters
M = 544;             % samples per symbol
nu = 32;             % cyclic prefix length
%load ../j02_merry/channels/csaloop1.time
load csaloop1.time
c = (csaloop1(:,2));
c = c/norm(c);       % unit norm channel
Lc = length(c)-1;    % channel length - 1
Lf = 15;             % equalizer length - 1
L = Lf+Lc;           % length of combined channel - 1
sx = 1;              % sigma_x
ssx = sx*sx;         % signal power
% Choose one pair below.
% The first pair runs 9 SNR values and plots performance vs. time
% for the fifth SNR value.
% The second pair runs a single SNR value.
%SNRs = [20:5:60];    % in dB
%plotme = 5;          % index of SNR for cost vs. time plots
SNRs = 40;           % in dB
plotme = 1;          % index of SNR for cost vs. time plots

% algorithm parameters
iter = 75;               % number of symbols (M samples each)
mu = 5;                  % SAM step size
offset = Lf+1+L;         % data averaging window location
N = 32;                  % data averaging window size
alpha = 1/100;           % AR implementation of summation
ARon = 1;                % flag: 0 = MA, 1 = AR implementation
cnst = 0;                % constraint (see samiterar.m)
finit = zeros(Lf+1,1);   % equalizer initialization
finit(ceil(Lf/2)) = 1;   %   (single spike)

% maximum shortening SNR solution from UT Austin's toolbox
%   (the solution does not depend on the noise,
%    although the performance does)
[fmelsa,dopt_mel,dummy] = mssnr(c,nu+1,Lf+1,1,L-nu-1,0);
hmel1 = conv(c,fmelsa).';
hmel = hmel1/norm(hmel1);   % scale it to unit norm
Rhh = conv(hmel,fliplr(hmel));
Rhh = Rhh((length(Rhh)+1)/2 : end);  % make it one-sided
Jsam_mel = sum( (Rhh(nu+1 +1 : end)).^2 );
Jssnr_mel = 1/ssnr2(hmel,nu);
%finit = fmelsa;

fmtx = zeros(Lf+1,length(SNRs));
Jsammtx = zeros(1,length(SNRs));
Jssnrmtx = zeros(1,length(SNRs));
Jbrmtx = zeros(1,length(SNRs));
BR_mel = zeros(1,length(SNRs));
BR_MFB = zeros(1,length(SNRs));

for SNRind = 1:length(SNRs)
%--------------------------------------------------
SNR = SNRs(SNRind)
g = 10.^(-SNR/10);   % 1/SNR in linear scale
sn = sx*sqrt(g);     % sigma_n
ssn = sn*sn;         % noise power

% generate ADSL received data
r = adslout(c,sx,sn,iter);

% only do lengthy cost calculations for one SNR value
if SNRind == plotme
   flag1 = 1;
else
   flag1 = 0;
end

% run SAM to get equalizer and shortened channel
if ARon
   params = [mu,alpha,nu,Lc,cnst,sx,sn,flag1];
   % auto-regressive estimation of expectations in update
   [f,Jsam,Jssnr,BR] = samiterar(finit,r,c,params);
else
   params = [offset,mu,N,nu,Lc,cnst,sx,sn,flag1];
   % block (summation) estimation of expectations in update
   [f,Jsam,Jssnr,BR] = samiter(finit,r,c,params);
end
kmax = length(Jsam);

if SNRind == plotme
   JsamP = Jsam;
   JssnrP = Jssnr;
   BRP = BR;
   comb = conv(c,f);
end

% save results
fmtx(:,SNRind) = f;
Jsammtx(SNRind) = Jsam(end);
Jssnrmtx(SNRind) = Jssnr(end);
Jbrmtx(SNRind) = BR(end);
BR_mel(SNRind) = sum(bitrate(hmel1.',fmelsa,nu,dopt_mel,ssx,ssn));
BR_MFB(SNRind) = sum(mfb(c,ssx,ssn));

%--------------------------------------------------
end  % SNR loop
clear f; clear Jsam; clear Jssnr; clear BR;

% print performance results
SNRs
Jsammtx
Jssnrmtx
Jbrmtx/(246.4)  % in Mbps
BR_mel/(246.4)  % in Mbps
BR_MFB/(246.4)  % in Mbps

% string for x-axis label
if ARon
   xstr = 'sample number, n';
else
   xstr = 'averaging block number, k';
end

% provide plots for SNR value number "plotme"

% plot SAM cost vs. time
figure(1);
clf;
set(0,'DefaultAxesFontSize',16);
semilogy(1:kmax,JsamP,'b-','LineWidth',2);
title('SAM cost vs. iteration number','FontSize',16);
xlabel(xstr,'FontSize',16);
ylabel('J_S_A_M','FontSize',16);
axis([0 kmax 1e-11 1e-4]);
grid on

% plot 1/SSNR vs. time
figure(2);
clf;
set(0,'DefaultAxesFontSize',16);
semilogy(1:kmax,JssnrP,'b-',...
   [0,kmax],Jssnr_mel*[1,1],'k--','LineWidth',2);
title('1/SSNR cost vs. iteration number','FontSize',16);
xlabel(xstr,'FontSize',16);
ylabel('1/SSNR','FontSize',16);
legend('SAM','MSSNR',0);
axis([0 kmax 1e-7 1e-1]);
grid on

% plot bit rate vs. time
Ts = 246.4 * 10^(-6);  % symbol duration, including CP
figure(3);
clf;
set(0,'DefaultAxesFontSize',16);
plot(1:kmax,BRP/Ts,'b-',...
   [0:kmax/10:kmax],BR_mel(plotme)/Ts*ones(11,1),'k--',...
   [0:kmax/10:kmax],BR_MFB(plotme)/Ts*ones(11,1),'kd-',...
   'LineWidth',2);
%plot([1:kmax]/M,BRP/Ts,'b-',...
%   [0:kmax/10:kmax]/M,BR_mel(plotme)/Ts*ones(11,1),'k--',...
%   [0:kmax/10:kmax]/M,BR_MFB(plotme)/Ts*ones(11,1),'kd-',...
%   'LineWidth',2);
title('bit rate vs. iteration number','FontSize',16);
xlabel(xstr,'FontSize',16);  
%xlabel('symbol number','FontSize',16);  
ylabel('bits per second','FontSize',16);
legend('SAM','MSSNR','MFB',0);
yax = axis;
axis([0 kmax 0 yax(4)]);
%axis([0 kmax/M 0 yax(4)]);
grid on

% plot equalizer and shortened channel impulse responses
figure(4);
clf
set(0,'DefaultAxesFontSize',16);
plot(0:Lc,c,'r--', 0:L,comb,'b-','LineWidth',2);
title('Results of SAM on CSA loop 1','FontSize',16);
xlabel('tap number','FontSize',16);
ylabel('tap values','FontSize',16);
legend('channel','shortened channel',1);
axis([0 200 -0.15 0.35]);
grid on

% plot bit rates vs. SNR
figure(5);
clf
set(0,'DefaultAxesFontSize',16);
plot(SNRs,Jbrmtx/Ts,'b-', SNRs,BR_mel/Ts,'k--',...
   SNRs,BR_MFB/Ts,'kd-','LineWidth',2);
title('achievable bit rate','FontSize',16);
xlabel('SNR, in dB','FontSize',16);
ylabel('bits per second','FontSize',16);
legend('SAM','MSSNR','MFB',2);
grid on

% plot equalizers
figure(6);
clf
set(0,'DefaultAxesFontSize',16);
stem([0:Lf],fmtx(:,plotme));
hold on
stem([0:Lf]+0.25,fmelsa/norm(fmelsa),'filled');
hold off
title('TEQ taps','FontSize',16);
xlabel('tap number','FontSize',16);
ylabel('tap value','FontSize',16);
%legend('SAM','MSSNR',1);
grid on

% plot sam cost and bit rate vs. time
figure(7);
clf;
set(0,'DefaultAxesFontSize',16);
subplot(2,1,1);
semilogy(1:kmax,JsamP,'b-','LineWidth',2);
%title('SAM cost vs. iteration number','FontSize',16);
%xlabel(xstr,'FontSize',16);
ylabel('SAM cost','FontSize',16);
axis([0 kmax 1e-11 1e-4]); 
grid on
subplot(2,1,2);
plot(1:kmax,BRP/Ts/1e6,'b-',...
   [0:kmax/10:kmax],BR_mel(plotme)/Ts/1e6*ones(11,1),'k--',...
   [0:kmax/10:kmax],BR_MFB(plotme)/Ts/1e6*ones(11,1),'kd-',...
   'LineWidth',2);
%title('bit rate vs. iteration number','FontSize',16);
xlabel(xstr,'FontSize',16);
ylabel('bit rate (Mbps)','FontSize',16);
legend('SAM','MSSNR','MFB',0);
yax = axis;
axis([0 kmax 0 yax(4)]);
grid on
